#include "iface.h"
#include "grint.h"
#include "allnodes.h"
#include <math.h>


 typedef struct
   {
   double  x;
   double  y;
   } XYPOS;


 typedef struct
   {
   XYPOS  allpos [8];
   double  f1;
   double  f2;
   } PARMS;


 void ball (int id, double x, double y, double vx, double vy)
   {
   int        i, color;
   FILLSTYLE  F;
   K_MSG      M;
   PARMS      parms;
   XYPOS      mypos;
   double      fx, fy, m;
   int colors [9] =
     {
     BLACK,
     LIGHTGREEN,
     LIGHTRED,
     LIGHTMAGENTA,
     YELLOW,
     LIGHTBLUE,
     GREEN,
     LIGHTCYAN,
     MAGENTA
     };

   color = colors [KS_NodeId >> 16];

   KS_LockW (GRAPHRES);
   while (1)
     {
     /* Plot ball at current position */

     F.type  = SOLID_FILL;
     F.color = color;
     setfillstyle (&F);
     setcolor (WHITE);
     fillellipse ((int) x + 320, (int) y + 240, 6, 6);
     KS_Unlock (GRAPHRES);

     /* Report current posiotion to master task */

     mypos.x = x;
     mypos.y = y;
     M.size = sizeof (XYPOS);
     M.info = id;
     M.tx_data = &mypos;
     M.rx_task = MASTER;
     KS_SendW (REPORT, 1, &M);

     /* Wait for message from master, giving current
        positions for all balls and parameters */

     M.size = sizeof (PARMS);
     M.rx_data = &parms;
     M.tx_task = MASTER;
     KS_ReceiveW (GETPOS, &M);

     /* Compute mutual forces */

     fx = fy = 0.0;
     for (i = 0; i < 8; i++)
       {
       double dx, dy;
       if (i == id) continue;
       dx = parms.allpos [i].x - x;
       dy = parms.allpos [i].y - y;
       m = dx * dx + dy * dy + 1;
       fx += dx / m;
       fy += dy / m;
       }
     fx *= parms.f1;
     fy *= parms.f1;

     /* Add 'magnetic field' force */

     fx += parms.f2 * vy;
     fy -= parms.f2 * vx;

     /* Calculate new speed vector */

     vx += fx;
     vy += fy;

     /* Limit speed to 30.0 */

     m = sqrt (vx * vx + vy * vy);
     if (m > 30.0)
       {
       m = 30.0 / m;
       vx *= m;
       vy *= m;
       }

     /* Calculate new position */

     x += vx;
     y += vy;

     /* Bounce ball on screen limits */

     do
       {
       i = 0;
       if (x < -310.0) { x = -620.0 - x; vx = -vx; i++; }
       if (x  > 310.0) { x =  620.0 - x; vx = -vx; i++; }
       if (y < -230.0) { y = -460.0 - y; vy = -vy; i++; }
       if (y >  230.0) { y =  460.0 - y; vy = -vy; i++; }
       }
     while (i);

     /* Unplot ball at previous position */

     F.color = DARKGRAY;
     F.type  = SOLID_FILL;
     KS_LockW (GRAPHRES);
     setfillstyle (&F);
     setcolor (BLACK);
     fillellipse ((int) mypos.x + 320, (int) mypos.y + 240, 6, 6);
     }
   }



