#include "iface.h"
#include "node1.h"


 static char string [80];


#define N_MONKS   20    /* the number of monks   */
#define N_SINNERS  7    /* the number of sinners */
#define TRUE       1
#define FALSE      0
#define DONTKNOW   0


 static int Dining [N_MONKS];
 static int Marked [N_MONKS];
 
/*
 Marked [x] is TRUE iff monk x is marked with a cross.
 Dining [x] is TRUE iff monk x is present at dinner.
 Monks only read these arrays be using the two functions
 defined below. Count_Dining () counts how many monks are
 present at dinner. Count_Marked (x) counts how many monks 
 have the mark, as seen by monk x.
*/

 static int Count_Dining (void)
   {
   int i, n;
   for (i = n = 0; i < N_MONKS; i++)
     {
     if (Dining [i]) n++;
     }
   return n;
   }

 static int Count_Marked (int ThisMonk)
   {
   int i, n;
   for (i = n = 0; i < N_MONKS; i++)
     {
     if (   i != ThisMonk
         && Dining [i]
         && Marked [i]) n++;
     }
   return n;
   }


 static void Monk (int ThisMonk)
   {
/*
  This function simulates one monk. Since all monks are identical,
  we can run N_MONKS instances of the same function. The simulation
  starts just after the appearance of the Spirit of Logic. At that
  time, all monks know there is at least one sinner.
*/
  int I_Have_Sinned = DONTKNOW;
  int Minimum       = 1;
  int AllMonks      = Count_Dining ();
/*
  When you know there are more sinners than the number of marks
  you can see, you can no longer deny your sins.
*/
  if (Count_Marked (ThisMonk) < Minimum) I_Have_Sinned = TRUE;

  while (1) /* Life goes on ... */
    {
/*
    We keep the monks synchronised by making them wait for the
    two great events of each day : waking up and going to dinner.
    Since these two events alternate, we can use the same two
    semaphores for all monks - we'll just signal N_MONKS times.
*/
    KS_Wait (MORNING);
/*
    Think and wait till dinner time.
*/
    KS_Wait (DINNER);
/*
    If you have sinned, stay in your cell and be forgiven.
    Otherwise, go to dinner. If all are present, then they all
    counted at least Minimum marks yesterday. But this implies
    there must be one more.
*/

    if (I_Have_Sinned)
      {
      Dining [ThisMonk] = FALSE;
      Marked [ThisMonk] = FALSE;
      }
    else
      {
      Dining [ThisMonk] = TRUE;
      KS_Sleep (100); /* wait until all have arrived */
      if (Count_Dining () == AllMonks)
	{
	Minimum++;
        if (Count_Marked (ThisMonk) < Minimum) I_Have_Sinned = TRUE;
        }
      }

    }
  }

/* TASKS */

 void Monk0  (void) { Monk (0); }
 void Monk1  (void) { Monk (1); }
 void Monk2  (void) { Monk (2); }
 void Monk3  (void) { Monk (3); }
 void Monk4  (void) { Monk (4); }
 void Monk5  (void) { Monk (5); }
 void Monk6  (void) { Monk (6); }
 void Monk7  (void) { Monk (7); }
 void Monk8  (void) { Monk (8); }
 void Monk9  (void) { Monk (9); }
 void Monk10 (void) { Monk (10); }
 void Monk11 (void) { Monk (11); }
 void Monk12 (void) { Monk (12); }
 void Monk13 (void) { Monk (13); }
 void Monk14 (void) { Monk (14); }
 void Monk15 (void) { Monk (15); }
 void Monk16 (void) { Monk (16); }
 void Monk17 (void) { Monk (17); }
 void Monk18 (void) { Monk (18); }
 void Monk19 (void) { Monk (19); }

/*
   The master task marks the sinners, keeps time,
   and watches the number present at dinner.
*/

 void Master (void)
   {
   int i, n, day;

   for (i = 0; i < N_MONKS; i++)
     {
     Dining [i] = TRUE;
     Marked [i] = (i < N_SINNERS) ? TRUE : FALSE;
     }

   printl ("The Monks riddle -- simulated using VIRTUOSO\n", CONRES, CONOQ);
   printl ("See ISI NEWS #10 for details\n\n", CONRES, CONOQ);

   day = 1;
   while (TRUE)
     {
     KS_Sleep (1000);

     sprintf (string, "Day %2d, morning\n", day);
     printl (string, CONRES, CONOQ);
     for (i = 0; i < N_MONKS; i++) KS_Signal (MORNING);

     KS_Sleep (1000);

     sprintf (string, "Day %2d, dinner - ", day);
     printl (string, CONRES, CONOQ);
     for (i = 0; i < N_MONKS; i++) KS_Signal (DINNER);

     KS_Sleep (100); /* wait until all diners are present */

     n = Count_Dining ();
     if (n == N_MONKS) printl ("all monks present\n", CONRES, CONOQ);
     else
       {
       sprintf (string, "%d missing\n", N_MONKS - n);
       printl (string, CONRES, CONOQ);
       break;
       }
     day++;
     }

   KS_Sleep (300);
   server_terminate ();
   }
