/*
 ----------------------------------------------------------------------

   VIRTUOSO CLASSICO

   $Id: global.h 1.3 94/05/16 11:00:44 FA REL3093 Locker: FA $

   Copyright (c) 1990-93
   Intelligent Systems International nv/sa

   Lindestraat 9, B-3210 Linden
   Belgium

   Tel +32 16.62.15.85
   Fax +32 16.62.15.84

 ----------------------------------------------------------------------
*/


#ifndef GLOBAL_H
#define GLOBAL_H


 extern K_NODE    K_ThisNode;
 extern K_PROC    K_TaskList [];
 extern K_TQHD    K_PrioList [];

 extern int       K_NodeCount;
 extern int       K_TaskCount;
 extern int       K_PrioCount;

 extern MAP_STRUCT K_MapList [];
 extern MBX_STRUCT K_MbxList [];
 extern RES_STRUCT K_ResList [];
 extern SEM_STRUCT K_SemList [];
 extern QUE_STRUCT K_QueList [];

 extern int       K_MapCount;
 extern int       K_MbxCount;
 extern int       K_ResCount;
 extern int       K_SemCount;
 extern int       K_QueCount;

 extern int       K_DataSize;
 extern int       K_DataNall;
 extern int       K_ArgsNall;
 extern int       K_TimerNall;

 extern int       ticktime;
 extern int       tickfreq;
 extern int       tickunit;

 extern int       ExtLinkInN;
 extern int       ExtLinkOutN;
 extern Channel **ExtLinkInMap;
 extern Channel **ExtLinkOutMap;

 extern K_PROC   *K_Task;
 extern K_TQHD   *K_Prio;
 extern K_TIMER  *K_Thead;
 extern K_TIMER  *K_Ttail;
 extern int       K_LowTime;

 extern _ALIST    KernArgs;
 extern Channel   TimerChan;
 extern Channel   EventChan;
 extern Channel   LinkIn  [4];
 extern Channel   LinkOut [4];

 extern void     *K_DataFreeP;
 extern K_ARGS   *K_ArgsFreeP;
 extern K_TIMER  *K_TimerFreeP;

 extern void     *K_GetData (void);
 extern K_ARGS   *K_GetArgs (void);
 extern void      K_FreeData (void *);
 extern void      K_FreeArgs (K_ARGS *);

 extern void      set_state_bit (K_PROC *, int);
 extern void      reset_state_bit (K_PROC *, int);

 extern void      delist_timeout (K_TIMER *T);
 extern void      enlist_timeout (K_ARGS *A);
 extern void      force_timeout (K_ARGS *A);

 extern void      SendPacket (K_ARGS *A);
 extern int       KS_CallKernel (K_ARGS *A);

 extern void      init_drivers (void);


/* Task status flags */

#define TF_STOP    0x00000001
#define TF_TERM    0x00000002
#define TF_SUSP    0x00000004
#define TF_BLCK    0x00000008
#define TF_TIME    0x00000010
#define TF_DRIV    0x00000020
#define TF_DATA    0x00000040
#define TF_EVNT    0x00000080
#define TF_ENQU    0x00000100
#define TF_DEQU    0x00000200
#define TF_SEND    0x00000400
#define TF_RECV    0x00000800
#define TF_SEMA    0x00001000
#define TF_LIST    0x00002000
#define TF_LOCK    0x00004000
#define TF_ALLO    0x00008000
#define TF_TEMP    0x01000000
#define TF_NETW    0x02000000
#define TF_ALLW    0x03FFFFF0


#ifdef TLDEBUG

 extern int    K_DebugHalt;
 extern char  *K_NodeNames [];
 extern char **K_TaskNames [];
 extern char **K_MapNames [];
 extern char **K_MbxNames [];
 extern char **K_ResNames [];
 extern char **K_SemNames [];
 extern char **K_QueNames [];

#define TASK_INFO     1
#define WSP_INFO      2
#define MAP_INFO      3
#define MBX_INFO      4
#define QUE_INFO      5
#define RES_INFO      6
#define SEM_INFO      7
#define TIMER_INFO    8
#define APCK_INFO     9
#define DPCK_INFO    10
#define RESET_COUNTS 11
#define MONIT_INFO   12

#endif

#ifdef TLMONIT

 extern void   K_monitor_init (void);
 extern void   K_monitor_task (K_PROC *, UNS32 d2);
 extern void   K_monitor_args (K_ARGS *);
 extern void   K_monitor_read (K_ARGS *);
 extern int    K_monitor_size;                /* defined in MAIN*.C */
 extern int    K_monitor_mask;                /* defined in MAIN*.C */

#define MO_STBIT0 0x20000000
#define MO_STBIT1 0x30000000
#define MO_EVENT  0x40000000
#define MO_LCOMM  0x50000000
#define MO_RCOMM  0x60000000

#endif


#define INSERT_ELM(L,E)              \
 {                                   \
 K_ARGS *X = (L);                    \
 K_ARGS *Y = NULL;                   \
 while (X && (X->Prio <= (E)->Prio)) \
   {                                 \
   Y = X;                            \
   X = X->Forw;                      \
   }                                 \
 if (Y) Y->Forw = (E);               \
 else       (L) = (E);               \
 (E)->Forw = X;                      \
 (E)->Head = &(L);                   \
 }

#define REMOVE_ELM(E)           \
 {                              \
 K_ARGS *X = *((E)->Head);      \
 K_ARGS *Y = NULL;              \
                                \
 while (X && (X != (E)))        \
   {                            \
   Y = X;                       \
   X = X->Forw;                 \
   }                            \
 if (X)                         \
   {                            \
   if (Y)    Y->Forw = X->Forw; \
   else *((E)->Head) = X->Forw; \
   }                            \
 }


#define GETARGS(A)                                 \
 {                                                 \
 while (! K_ArgsFreeP) {__asm { ldtimer; tin; } }  \
 A = K_ArgsFreeP;                                  \
 K_ArgsFreeP = (A)->Forw;                          \
 }

#define GETTIMER(T)                                \
 {                                                 \
 while (! K_TimerFreeP) {__asm { ldtimer; tin; } } \
 T = K_TimerFreeP;                                 \
 K_TimerFreeP = (T)->Forw;                         \
 }

#define GETDATA(D)                                 \
 {                                                 \
 while (! K_DataFreeP) {__asm { ldtimer; tin; } }  \
 D = K_DataFreeP;                                  \
 K_DataFreeP = *(void **)(D);                      \
 }

#define FREEARGS(A)  \
 { (A)->Forw = K_ArgsFreeP;  K_ArgsFreeP = (A);  }

#define FREETIMER(T) \
 { (T)->Forw = K_TimerFreeP; K_TimerFreeP = (T); }

#define FREEDATA(P)  \
 { void *T = P; *(void **)T = K_DataFreeP; K_DataFreeP = T; }


#define TO_ALIST(L,A)                      \
  {                                        \
  if ((L).Chan != (void *) NotProcess_p)   \
    {                                      \
    ChanOutInt (&((L).Chan), (int)(A));    \
    }                                      \
  else                                     \
    {                                      \
    *(L.Lptr) = A;                         \
    L.Lptr++;                              \
    }                                      \
  }


#endif



