/*
 ----------------------------------------------------------------------

   VIRTUOSO CLASSICO

   $Id: grint.h 1.3 94/05/16 11:00:49 FA REL3093 Locker: FA $

   Copyright (c) 1990-93
   Intelligent Systems International nv/sa

   Lindestraat 9, B-3210 Linden
   Belgium

   Tel +32 16.62.15.85
   Fax +32 16.62.15.84

 ----------------------------------------------------------------------
*/


/* GRINT.H - task level interface to distributes grahics services */


#ifndef BGIDRV_H
#define BGIDRV_H

#include "bgitags.h"


#define HORIZ_DIR	0	/* left to right */
#define VERT_DIR	1	/* bottom to top */
#define USER_CHAR_SIZE	0	/* user-defined char size */
#define MAXCOLORS 15


 enum graphics_errors
   {		       /* graphresult error return codes */
   grOk		     =   0,
   grNoInitGraph     =  -1,
   grNotDetected     =  -2,
   grFileNotFound    =  -3,
   grInvalidDriver   =  -4,
   grNoLoadMem	     =  -5,
   grNoScanMem	     =  -6,
   grNoFloodMem	     =  -7,
   grFontNotFound    =  -8,
   grNoFontMem	     =  -9,
   grInvalidMode     = -10,
   grError 	     = -11,   /* generic error */
   grIOerror	     = -12,
   grInvalidFont     = -13,
   grInvalidFontNum  = -14,
   grInvalidVersion  = -18
   };

 enum graphics_drivers
   {                    /* define graphics drivers */
   DETECT, 		/* requests autodetection */
   CGA,
   MCGA,
   EGA,
   EGA64,
   EGAMONO,
   IBM8514,
   HERCMONO,
   ATT400,
   VGA,
   PC3270,
   CURRENT_DRIVER = -1
   };

 enum graphics_modes
   {                   /* graphics modes for each driver */
   CGAC0      = 0,  /* 320x200 palette 0; 1 page	*/
   CGAC1      = 1,  /* 320x200 palette 1; 1 page	*/
   CGAC2      = 2,  /* 320x200 palette 2: 1 page	*/
   CGAC3      = 3,  /* 320x200 palette 3; 1 page	*/
   CGAHI      = 4,  /* 640x200 1 page			*/
   MCGAC0     = 0,  /* 320x200 palette 0; 1 page	*/
   MCGAC1     = 1,  /* 320x200 palette 1; 1 page	*/
   MCGAC2     = 2,  /* 320x200 palette 2; 1 page	*/
   MCGAC3     = 3,  /* 320x200 palette 3; 1 page	*/
   MCGAMED    = 4,  /* 640x200 1 page			*/
   MCGAHI     = 5,  /* 640x480 1 page			*/
   EGALO      = 0,  /* 640x200 16 color 4 pages	*/
   EGAHI      = 1,  /* 640x350 16 color 2 pages	*/
   EGA64LO    = 0,  /* 640x200 16 color 1 page 	*/
   EGA64HI    = 1,  /* 640x350 4 color  1 page 	*/
   EGAMONOHI  = 0,  /* 640x350 64K on card, 1 page - 256K on card, 4 pages */
   HERCMONOHI = 0,  /* 720x348 2 pages 		*/
   ATT400C0   = 0,  /* 320x200 palette 0; 1 page	*/
   ATT400C1   = 1,  /* 320x200 palette 1; 1 page	*/
   ATT400C2   = 2,  /* 320x200 palette 2; 1 page	*/
   ATT400C3   = 3,  /* 320x200 palette 3; 1 page	*/
   ATT400MED  = 4,  /* 640x200 1 page			*/
   ATT400HI   = 5,  /* 640x400 1 page			*/
   VGALO      = 0,  /* 640x200 16 color 4 pages	*/
   VGAMED     = 1,  /* 640x350 16 color 2 pages	*/
   VGAHI      = 2,  /* 640x480 16 color 1 page 	*/
   PC3270HI   = 0,  /* 720x350 1 page			*/
   IBM8514LO  = 0,  /* 640x480 256 colors		*/
   IBM8514HI  = 1   /*1024x768 256 colors		*/
   };


 /* Colors for setpalette and setallpalette */

 #ifndef __COLORS
 #define __COLORS

 enum COLORS
   {
   BLACK,		    /* dark colors */
   BLUE,
   GREEN,
   CYAN,
   RED,
   MAGENTA,
   BROWN,
   LIGHTGRAY,
   DARKGRAY,		    /* light colors */
   LIGHTBLUE,
   LIGHTGREEN,
   LIGHTCYAN,
   LIGHTRED,
   LIGHTMAGENTA,
   YELLOW,
   WHITE
   };
 #endif

 enum CGA_COLORS
   {
   CGA_LIGHTGREEN     = 1,		/* Palette C0 Color Names	*/
   CGA_LIGHTRED       = 2,
   CGA_YELLOW	      = 3,
   CGA_LIGHTCYAN      = 1,		/* Palette C1 Color Names	*/
   CGA_LIGHTMAGENTA   = 2,
   CGA_WHITE	      = 3,
   CGA_GREEN	      = 1,		/* Palette C2 Color Names	*/
   CGA_RED	      = 2,
   CGA_BROWN	      = 3,
   CGA_CYAN	      = 1,		/* Palette C3 Color Names	*/
   CGA_MAGENTA        = 2,
   CGA_LIGHTGRAY      = 3
   };

 enum EGA_COLORS
   {
   EGA_BLACK		 =  0,	    /* dark colors */
   EGA_BLUE		 =  1,
   EGA_GREEN		 =  2,
   EGA_CYAN		 =  3,
   EGA_RED		 =  4,
   EGA_MAGENTA 	         =  5,
   EGA_BROWN		 =  20,
   EGA_LIGHTGRAY	 =  7,
   EGA_DARKGRAY	         =  56,     /* light colors */
   EGA_LIGHTBLUE	 =  57,
   EGA_LIGHTGREEN	 =  58,
   EGA_LIGHTCYAN	 =  59,
   EGA_LIGHTRED	         =  60,
   EGA_LIGHTMAGENTA	 =  61,
   EGA_YELLOW		 =  62,
   EGA_WHITE		 =  63
   };

 enum line_styles
   {	               	 /* Line styles for get/setlinestyle */
   SOLID_LINE   = 0,
   DOTTED_LINE  = 1,
   CENTER_LINE  = 2,
   DASHED_LINE  = 3,
   USERBIT_LINE = 4	 /* User defined line style */
   };

 enum line_widths
   {                     /* Line widths for get/setlinestyle */
   NORM_WIDTH  = 1,
   THICK_WIDTH = 3
   };

 enum font_names
   {
   DEFAULT_FONT    = 0,  /* 8x8 bit mapped font */
   TRIPLEX_FONT    = 1,  /* "Stroked" fonts */
   SMALL_FONT      = 2,
   SANS_SERIF_FONT = 3,
   GOTHIC_FONT	   = 4
   };

 enum fill_patterns
   {                     /* Fill patterns for get/setfillstyle */
   EMPTY_FILL,		 /* fills area in background color */
   SOLID_FILL,		 /* fills area in solid fill color */
   LINE_FILL,		 /* --- fill */
   LTSLASH_FILL,	 /* /// fill */
   SLASH_FILL,		 /* /// fill with thick lines */
   BKSLASH_FILL,	 /* \\\ fill with thick lines */
   LTBKSLASH_FILL, 	 /* \\\ fill */
   HATCH_FILL,		 /* light hatch fill */
   XHATCH_FILL,		 /* heavy cross hatch fill */
   INTERLEAVE_FILL,	 /* interleaving line fill */
   WIDE_DOT_FILL,	 /* Widely spaced dot fill */
   CLOSE_DOT_FILL, 	 /* Closely spaced dot fill */
   USER_FILL		 /* user defined fill */
   };

 enum putimage_ops
   {                     /* BitBlt operators for putimage */
   COPY_PUT,		 /* MOV */
   XOR_PUT,		 /* XOR */
   OR_PUT, 		 /* OR  */
   AND_PUT,		 /* AND */
   NOT_PUT 		 /* NOT */
   };

 enum text_just
   {                     /* Horizontal and vertical text justification */
   LEFT_TEXT	= 0,
   CENTER_TEXT	= 1,
   RIGHT_TEXT	= 2,
   BOTTOM_TEXT	= 0,
/* CENTER_TEXT	= 1,  already defined above */
   TOP_TEXT	= 2
   };



 typedef struct modepars
   {
   int mode;
   int maxcol;
   int maxx;
   int maxy;
   int xasp;
   int yasp;
   } MODEPARS;

 typedef struct userpars
   {
   int color;
   int bkcolor;
   int linestyle;
   int pattern;
   int thickness;
   int font;
   int direction;
   int charsize;
   int horiz;
   int vert;
   } USERPARS;

 typedef struct viewport
   {
   int left;
   int top;
   int right;
   int bottom;
   int clip;
   } VIEWPORT;

 typedef struct palette
   {
   char colors [16];
   } PALETTE;

 typedef struct fillstyle
   {
   int  type;
   int  color;
   char pattern [8];
   } FILLSTYLE;

 typedef struct point
   {
   int x;
   int y;
   } POINT;

 typedef struct arccoords
   {
   int xcentre;
   int ycentre;
   int xstart;
   int ystart;
   int xend;
   int yend;
   } ARCCOORDS;


 extern void detectgraph (int *driver, int *mode);
 extern void initgraph (int driver, int mode);
 extern void setgraphmode (int mode);
 extern void getmodepars (struct modepars *mpars);
 extern int  installuserfont (char *name);
 extern void graphdefaults (void);
 extern void setactivepage (int page);
 extern void setvisualpage (int page);
 extern void closegraph (void);
 extern void restorecrtmode (void);
 extern int  savescreen (void);
 extern int  graphresult (void);

 extern void getuserpars (struct userpars *upars);

 extern void getviewport (struct viewport *vport);
 extern void setviewport (struct viewport *vport);
 extern void cleardevice (void);
 extern void clearviewport (void);

 extern void getallpalette (struct palette *pal);
 extern void setallpalette (struct palette *pal);
 extern void setpalette (int colornum, int color);
 extern void setrgbpalette (int color, int red, int green, int blue);

 extern void getfillstyle (struct fillstyle *fstyle);
 extern void setfillstyle (struct fillstyle *fstyle);

 extern void getcurrcoords (struct point *xy);
 extern void getarccoords (struct arccoords *arc);

 extern void setbkcolor (int color);
 extern void setcolor (int color);
 extern void setlinestyle (int linestyle, int pattern, int thickness);
 extern void setwritemode (int mode);

 extern void putpixel (int x, int y, int color);
 extern int  getpixel (int x, int y);
 extern void linerel (int dx, int dy);
 extern void moverel (int dx, int dy);
 extern void lineto (int x, int y);
 extern void moveto (int x, int y);
 extern void line (int x1, int y1, int x2, int y2);
 extern void rectangle (int left, int top, int right, int bottom);
 extern void circle (int x, int y, int radius);
 extern void ellipse (int x, int y, int a0, int a1, int xr, int yr);
 extern void arc (int x, int y, int a0, int a1, int r);
 extern void pieslice (int x, int y, int a0, int a1, int r);
 extern void sector (int x, int y, int a0, int a1, int xr, int yr);
 extern void fillellipse (int x, int y, int xr, int yr);
 extern void bar (int left, int top, int right, int bottom);
 extern void bar3d (int left, int top, int right, int bottom, int depth, int topflag);
 extern void drawpoly (int n, int *points);
 extern void fillpoly (int n, int *points);
 extern void floodfill (int x, int y, int border);

 extern void settextjustify (int horiz, int vert);
 extern void settextstyle (int font, int direction, int charsize);
 extern void setusercharsize (int multx, int divx, int multy, int divy);
 extern void textdimensions (char *text, int *x, int *y);
 extern void outtext (char *text);
 extern void outtextxy (int x, int y, char *text);

 extern int  getimage (int left, int top, int right, int bottom);
 extern void putimage (int left, int top, int area, int op);
 extern void freeimage (int area);


/*
  Functions having the same type and argument signature
  are mapped (by the #defines below) onto one of these :
*/

 extern void bgifunc00 (int t);
 extern void bgifunc01 (int t, int a);
 extern void bgifunc02 (int t, int a, int b);
 extern void bgifunc03 (int t, int a, int b, int c);
 extern void bgifunc04 (int t, int a, int b, int c, int d);
 extern void bgifunc05 (int t, int a, int b, int c, int d, int e);
 extern void bgifunc06 (int t, int a, int b, int c, int d, int e, int f);
 extern void bgigetfunc (int t, void *d);
 extern void bgisetfunc (int t, void *d);
 extern void bgipolyfunc (int t, int n, int *d);



#define cleardevice() bgifunc00 (BGI_CLRDEVC)
#define clearviewport() bgifunc00 (BGI_CLRVIEW)
#define graphdefaults() bgifunc00 (BGI_GRDEFAULT)
#define closegraph() bgifunc00 (BGI_CLSGRAPH)
#define restorecrtmode() bgifunc00 (BGI_RESTMODE)

#define setgraphmode(a) bgifunc01 (BGI_GRMODE, a)
#define setactivepage(a) bgifunc01 (BGI_ACTPAGE, a)
#define setvisualpage(a) bgifunc01 (BGI_VISPAGE, a)
#define setbkcolor(a) bgifunc01 (BGI_BACKCOL, a)
#define setcolor(a) bgifunc01 (BGI_FORECOL, a)
#define setwritemode(a) bgifunc01 (BGI_WRIMODE, a)
#define freeimage(a) bgifunc01 (BGI_FREAREA, a)

#define linerel(a,b) bgifunc02 (BGI_LINEREL, a, b)
#define moverel(a,b) bgifunc02 (BGI_MOVEREL, a, b)
#define lineto(a,b) bgifunc02 (BGI_LINETO, a, b)
#define moveto(a,b) bgifunc02 (BGI_MOVETO, a, b)
#define settextjustify(a,b) bgifunc02 (BGI_SETTJFY, a, b)
#define setpalette(a,b) bgifunc02 (BGI_SETPALET, a, b)

#define putpixel(a,b,c) bgifunc03 (BGI_PUTPIXEL, a, b, c)
#define floodfill(a,b,c) bgifunc03 (BGI_FLOODFLL, a, b, c)
#define circle(a,b,c) bgifunc03 (BGI_PLCIRCLE, a, b, c)
#define settextstyle(a,b,c) bgifunc03 (BGI_SETTXTST, a, b, c)
#define setlinestyle(a,b,c) bgifunc03 (BGI_SETLNEST, a, b, c)

#define bar(a,b,c,d) bgifunc04 (BGI_PLOTBAR, a, b, c, d)
#define rectangle(a,b,c,d) bgifunc04 (BGI_PLOTRECT, a, b, c, d)
#define line(a,b,c,d) bgifunc04 (BGI_PLOTLINE, a, b, c, d)
#define setusercharsize(a,b,c,d) bgifunc04 (BGI_USRCSIZE, a, b, c, d)
#define fillellipse(a,b,c,d) bgifunc04 (BGI_FILLELL, a, b, c, d)
#define putimage(a,b,c,d) bgifunc04 (BGI_PUTIMGE, a, b, c, d)
#define setrgbpalette(a,b,c,d) bgifunc04 (BGI_RGBPAL, a, b, c, d)

#define arc(a,b,c,d,e) bgifunc05 (BGI_PLOTARC, a, b, c, d, e)
#define pieslice(a,b,c,d,e) bgifunc05 (BGI_PIESLICE, a, b, c, d, e)

#define sector(a,b,c,d,e,f) bgifunc06 (BGI_SECTOR, a, b, c, d, e, f)
#define ellipse(a,b,c,d,e,f) bgifunc06 (BGI_ELLIPS, a, b, c, d, e, f)
#define bar3d(a,b,c,d,e,f) bgifunc06 (BGI_BAR3DIM, a, b, c, d, e, f)

#define getmodepars(a) bgigetfunc (BGI_GETMDEP, a)
#define getuserpars(a) bgigetfunc (BGI_GETUSRP, a)
#define getviewport(a) bgigetfunc (BGI_GETVIEW, a)
#define getallpalette(a) bgigetfunc (BGI_GETAPAL, a)
#define getfillstyle(a) bgigetfunc (BGI_GETFILL, a)
#define getcurrcoords(a) bgigetfunc (BGI_GETCURR, a)
#define getarccoords(a) bgigetfunc (BGI_GETARCC, a)

#define setviewport(a) bgisetfunc (BGI_SETVIEW, a)
#define setallpalette(a) bgisetfunc (BGI_SETAPAL, a)
#define setfillstyle(a) bgisetfunc (BGI_SETFILL, a)

#define drawpoly(a,b) bgipolyfunc (BGI_DRAWPOLY, a, b)
#define fillpoly(a,b) bgipolyfunc (BGI_FILLPOLY, a, b)

#define restscreen() bgifunc00 (BGI_RESTSCR);

#endif

