/*
 ----------------------------------------------------------------------

   VIRTUOSO CLASSICO

   $Id: iface.h 1.3 94/05/16 11:00:58 FA REL3093 Locker: FA $

   Copyright (c) 1990-93
   Intelligent Systems International nv/sa

   Lindestraat 9, B-3210 Linden
   Belgium

   Tel +32 16.62.15.85
   Fax +32 16.62.15.84

 ----------------------------------------------------------------------
*/


#ifndef IFACE_H
#define IFACE_H

#include "k_struct.h"


/* ------------------------------------------------------------------ */
/*
   The following functions and variables should never be used directly,
   as they will probably be different in future releases. Always use the
   #defines listed below for future-proof source code.
*/

 extern void     KS__Task (K_TASK, int);
 extern void     KS__Group (int, int);

 extern void     KS__Alloc (K_MAP, void **, K_TICKS);
 extern void     KS__Dealloc (K_MAP, void **);

 extern int      KS__Lock (K_RES, K_TICKS);
 extern void     KS__Unlock (K_RES);

 extern int      KS__SendMess (K_MBOX, K_PRIO, K_MSG *, K_TICKS);
 extern int      KS__RecvMess (K_MBOX, K_MSG *, K_TICKS);
 extern void     KS__RecvData (K_MSG *);

 extern int      KS__Enqueue (K_QUEUE, void *, int, K_TICKS);
 extern int      KS__Dequeue (K_QUEUE, void *, int, K_TICKS);
 extern int      KS__Queue (K_QUEUE, int);

 extern int      KS__Wait (K_SEMA, K_TICKS);
 extern K_SEMA   KS__WaitM (K_SEMA *, K_TICKS);

 extern int      hostlink;
 extern int      KS__Linkin (int link, int length, void *addr, int time);
 extern int      KS__Linkout (int link, int length, void *addr, int time);

 extern K_PROC  *K_Task;
 extern K_NODE   K_ThisNode;


/* ------------------------------------------------------------------ */
/* TASK MANAGEMENT */

 extern void KS_SetEntry (K_TASK task, void (*func)(void));
 extern void KS_Aborted (void (*func)(void));
 extern void KS_SetPrio (K_TASK task, int prio);
 extern void KS_Yield (void);
 extern void KS_Sleep (K_TICKS time);

#define KS_Start(t)     KS__Task (t, 0)
#define KS_Abort(t)     KS__Task (t, 1)
#define KS_Suspend(t)   KS__Task (t, 2)
#define KS_Resume(t)    KS__Task (t, 3)

#define KS_StartG(g)    KS__Group (g, 0)
#define KS_AbortG(g)    KS__Group (g, 1)
#define KS_SuspendG(g)  KS__Group (g, 2)
#define KS_ResumeG(g)   KS__Group (g, 3)

#define KS_TaskId    (K_Task->Ident)
#define KS_TaskPrio  (K_Task->Prio)
#define KS_GroupId   (K_Task->Group)
#define KS_JoinG(g)  (K_Task->Group |= g)
#define KS_LeaveG(g) (K_Task->Group &= ~g)
#define KS_NodeId    (K_ThisNode)


/* ------------------------------------------------------------------ */
/* TIMER MANAGEMENT */


 extern K_TIMER *KS_AllocTimer (void);
 extern void     KS_DeallocTimer (K_TIMER *timer);
 extern void     KS_StopTimer (K_TIMER *timer);
 extern void     KS_StartTimer (K_TIMER *timer,
                                K_TICKS  delay,
                                K_TICKS  period,
                                K_SEMA   sema);
 extern void     KS_RestartTimer (K_TIMER *timer,
                                  K_TICKS  delay,
                                  K_TICKS  period);
 extern K_TICKS  KS_Elapse (K_TICKS *reftime);
 extern K_TICKS  KS_LowTimer (void);


/* ------------------------------------------------------------------ */
/* MEMORY MANAGEMENT */


 extern int KS_InqMap (K_MAP map);

#define KS_Alloc(m,p)      KS__Alloc(m, p, 0)
#define KS_AllocW(m,p)     KS__Alloc(m, p, -1)
#define KS_AllocWT(m,p,t)  KS__Alloc(m, p, t)
#define KS_Dealloc(m,p)    KS__Dealloc(m, p)



/* ------------------------------------------------------------------ */
/* RESOURCE MANAGEMENT */


#define KS_Lock(r)        KS__Lock (r, 0)
#define KS_LockW(r)       KS__Lock (r, -1)
#define KS_LockWT(r,t)    KS__Lock (r, t)
#define KS_Unlock(r)      KS__Unlock (r)


/* ------------------------------------------------------------------ */
/* MESSAGES */


#define KS_Send(b,p,m)       KS__SendMess (b, p, m, 0)
#define KS_SendW(b,p,m)      KS__SendMess (b, p, m, -1)
#define KS_SendWT(b,p,m,t)   KS__SendMess (b, p, m, t)
#define KS_Receive(b,m)      KS__RecvMess (b, m, 0)
#define KS_ReceiveW(b,m)     KS__RecvMess (b, m, -1)
#define KS_ReceiveWT(b,m,t)  KS__RecvMess (b, m, t)
#define KS_ReceiveData(m)    KS__RecvData (m)


/* ------------------------------------------------------------------ */
/* QUEUES */


#define KS_Enqueue(q,p,s)       KS__Enqueue (q, p, s, 0)
#define KS_EnqueueW(q,p,s)      KS__Enqueue (q, p, s, -1)
#define KS_EnqueueWT(q,p,s,t)   KS__Enqueue (q, p, s, t)
#define KS_Dequeue(q,p,s)       KS__Dequeue (q, p, s, 0)
#define KS_DequeueW(q,p,s)      KS__Dequeue (q, p, s, -1)
#define KS_DequeueWT(q,p,s,t)   KS__Dequeue (q, p, s, t)
#define KS_InqQueue(q)		KS__Queue (q, 0)
#define KS_PurgeQueue(q)	KS__Queue (q, 1)


/* ------------------------------------------------------------------ */
/* SEMAPHORES */


 extern void KS_ResetSema (K_SEMA sema);
 extern void KS_ResetSemaM (K_SEMA *list);
 extern void KS_Signal (K_SEMA sema);
 extern void KS_SignalM (K_SEMA *list);
 extern int  KS_InqSema (K_SEMA sema);

#define KS_Wait(s)     KS__Wait (s, -1)
#define KS_WaitT(s,t)  KS__Wait (s, t)
#define KS_WaitM(s)    KS__WaitM (s, -1)
#define KS_WaitMT(s,t) KS__WaitM (s, t)


/* ------------------------------------------------------------------ */
/* SPECIAL SERVICES */


 extern void KS_Nop (void);
 extern int  KS_User (int (*)(),void *);
 extern int  KS_Workload (void);
 extern void KS_SetWlper (K_TICKS);


/* ------------------------------------------------------------------ */
/* DRIVERS & PROCESSOR SPECIFIC */


#define KS_LinkinW(c,l,d)     KS__Linkin  (c, l, d, 0)
#define KS_LinkoutW(c,l,d)    KS__Linkout (c, l, d, 0)
#define KS_LinkinWT(c,l,d,t)  KS__Linkin  (c, l, d, t)
#define KS_LinkoutWT(c,l,d,t) KS__Linkout (c, l, d, t)


 extern void KS_HostLink (int link, void *txdata, void *rxdata);
 extern void KS_EventW (void);
 extern int  KS_HighTimer (void);

 extern void NetLink_Driver (int link);
 extern void RawLink_Driver (int link);
 extern void HostLink_Driver (int link);
 extern void Event_Driver (void);

 extern void KS_MoveData (K_NODE N, int size, void *tx, void *rx, K_COMM comm);

/* these are defined in main#.c */

 extern int ticktime;
 extern int tickfreq;
 extern int tickunit;


/* for 'comm' in KS_MoveData, use one of these : */

#define SEND TXDATA_REQ
#define RECV RXDATA_REQ



/* ------------------------------------------------------------------ */
/* ODDS & ENDS */


#define EXE_GROUP 1
#define USR_GROUP 2
#define FPU_GROUP 4


 extern void call_server (void);
 extern void server_terminate (void);
 extern void server_echo (int n);
 extern void server_putchar (int c);
 extern void server_putstring (char *s);
 extern int  server_pollkey (void);
 extern int  server_pollesc (void);
 extern int  server_getarg (int i, char *s, int l);
 extern int  server_getenv (char *srce, char *dest, int l);
 extern int  server_system (char *comm);
 extern int  server_time (void); 

 extern void printl (char *, K_RES, K_QUEUE);

 extern int  netload (void);
 extern int  parsytec_netload (void);

 extern int  K_monitor_mask;
 extern int  ticktime;
 extern int  tickunit;
 extern int  tickfreq;

#define MON_TSWAP  1
#define MON_STATE  2
#define MON_KSERV  4
#define MON_EVENT  8
#define MON_ALL   15


/* ------------------------------------------------------------------ */


#endif


