/*
 ----------------------------------------------------------------------

   VIRTUOSO CLASSICO

   $Id: k_struct.h 1.3 94/05/16 11:01:04 FA REL3093 Locker: FA $

   Copyright (c) 1990-93
   Intelligent Systems International nv/sa

   Lindestraat 9, B-3210 Linden
   Belgium

   Tel +32 16.62.15.85
   Fax +32 16.62.15.84

 ----------------------------------------------------------------------
*/


#ifndef K_STRUCT_H
#define K_STRUCT_H

#include <stdlib.h>
#include <process.h>
#include <channel.h>
#include "k_types.h"

#define INTSAVEDLOC   0x8000002C

#define RC_OK   0
#define RC_FAIL 1
#define RC_TIME 2


 typedef struct k_args  K_ARGS;
 typedef struct k_proc  K_PROC;
 typedef struct k_tqhd  K_TQHD;
 typedef struct k_timer K_TIMER;


/* ----------------------------------------------------------------------*/
/* PUBLIC KERNEL OBJECT TYPES */


 typedef struct
   {
   INT32   size;          /* size of message (bytes)                */
   INT32   info;          /* information field, free for user       */
   void   *tx_data;       /* pointer to message data at sender side */
   void   *rx_data;       /* pointer to message data at receiver    */
   K_TASK  tx_task;       /* sending task                           */
   K_TASK  rx_task;       /* receiving task                         */
   K_ARGS *tx_wait;       /* internal use only                      */
   K_ARGS *rx_wait;       /* internal use only                      */
   } K_MSG;


 typedef struct k_timer   /* For pointer declarations only !!       */
   {                      /* Do not access fields in user code.     */
   K_TIMER   *Forw;
   K_TIMER   *Back;
   K_TICKS    Ti;
   K_TICKS    Tr;
   K_ARGS    *Args;
   };                     /* use K_TIMER as the timer type name     */


/* ----------------------------------------------------------------------*/
/* PROCESSOR SPECIFIC TYPES & DATA STRUCTURES */


/* Service codes */

 typedef enum
   {
   NOP,
   USER,
   READWL,
   SIGNALS,
   SIGNALM,
   RESETS,
   RESETM,
   WAITSREQ,
   WAITSRPL,
   WAITSTMO,
   WAITMANY,
   WAITMREQ,
   WAITMRDY,
   WAITMCAN,
   WAITMACC,
   WAITMEND,
   WAITMTMO,
   INQSEMA,
   LOCK_REQ,
   LOCK_RPL,
   LOCK_TMO,
   UNLOCK,
   ENQ_REQ,
   ENQ_RPL,
   ENQ_TMO,
   DEQ_REQ,
   DEQ_RPL,
   DEQ_TMO,
   QUEUE,
   SEND_REQ,
   SEND_RPL,
   SEND_TMO,
   SEND_ACK,
   RECV_REQ,
   RECV_RPL,
   RECV_TMO,
   RECV_ACK,
   ELAPSE,
   SLEEP,
   WAKEUP,
   TSKOP,
   GRPOP,
   SPRIO,
   YIELD,
   ALLOC,
   DEALLOC,
   TALLOC,
   TDEALLOC,
   TSTART,
   TSTOP,
   DRIVER_REQ,
   DRIVER_ACK,
   ALLOCTMO,
   REMREPLY,
   DEBUG_REQ,
   DEBUG_ACK,
   TXDATA_REQ,
   TXDATA_ACK,
   RXDATA_REQ,
   RXDATA_ACK,
   RAWDATA_REQ,
   RAWDATA_ACK,
   DATAWAIT,
   EVENTWAIT
   } K_COMM;


/* Transputer process queue header */

 typedef struct
   {
   void *F;
   void *B;
   } _PQHDR;


/* Transputer register set */

 typedef struct
   {
   int  W;
   int  I;
   int  A;
   int  B;
   int  C;
   int  S;
   int  E;
   } _FRAME;


/* Transputer fpu register set */

 typedef struct
   {
   int a [3];
   int b [3];
   int c [3];
   int e;
   } _FPREG;


/* Task queue header */

 typedef struct k_tqhd
   {
   K_PROC *Head;
   K_PROC *Tail;
   };


/* Task control block */

 typedef struct k_proc
   {
   K_PROC    *Forw;
   K_PROC    *Back;	/* NOT USED */
   K_PRIO     Prio;
   int        Ident;
   int        State;
   int        Group;
   void     (*fstart)(void);
   int       *workspace;
   int        worksize;
   void     (*fabort)(void);
   _FPREG    *fpreg;
   _FRAME     frame;
   _PQHDR     pqhdr;
   Process    Proc;
   K_ARGS    *Args;
   };


/* Monitor record */

 typedef struct k_mrec
   {
   UNS32   time;
   UNS32   data1;
   UNS32   data2;
   } K_MREC;


/* Command packet stack */

 typedef struct
   {
   Channel  Chan;
   K_ARGS **Lptr;
   K_ARGS  *List [64];
   } _ALIST;


/* Command packet list */

 typedef struct
   {
   Channel  Chan;
   K_ARGS  *List;
   } _SLIST;


/* ----------------------------------------------------------------------*/
/* COMMAND PACKETS */


 typedef union
   {
   K_TASK    task;
   K_PROC   *proc;
   K_ARGS   *args;
   } K_CREF;


 typedef struct
   {
   K_MAP     mmap;
   void    **mptr;
   } _A1ARG;


 typedef struct
   {
   K_TIMER  *timer;
   K_TICKS   time1;
   K_TICKS   time2;
   K_SEMA    sema;
   K_TASK    task;
   } _C1ARG;


 typedef struct
   {
   int       event;
   } _E1ARG;


 typedef struct
   {
   int       size;
   char     *txptr;
   char     *rxptr;
   K_ARGS   *txarg;
   K_ARGS   *rxarg;
   void     *data;
   int       todo;
   int       nack;
   } _D1ARG;


 typedef struct
   {
   K_TASK    task;
   K_TGROUP  group;
   int       prio;
   int       opt;
   int       val;
   } _G1ARG;


 typedef struct
   {
   K_RES     res;
   K_TASK    task;
   } _L1ARG;


 typedef struct
   {
   K_MSG     mess;
   } _M1ARG;


 typedef struct
   {
   K_QUEUE   queue;
   int       size;
   char	    *data;
   } _Q1ARG;


 typedef struct
   {
   K_QUEUE   queue;
   int       size;
   char	     data [24];
   } _Q2ARG;


 typedef struct
   {
   K_SEMA    sema;
   K_SEMA   *list;
   int       nsem;
   } _S1ARG;


 typedef struct
   {
   int     (*func)();
   void     *argp;
   int       rval;
   } _U1ARG;


 typedef struct
   {
   Channel  *chan;
   int       parm [7];
   } _X1ARG;


 typedef struct
   {
   int       Comm;
   int       Nitem;
   int       Laddr;
   int       Lsize;
   } _Z1ARG;


 typedef struct
   {
   int       Comm;
   K_TASK    Task;
   int       Wsize;
   int       Wused;
   int       Wmark;
   } _Z2ARG;


 typedef struct
   {
   int       Comm;
   int       Ind;
   int       Ti;
   int       Tr;
   K_ARGS   *Args;
   } _Z3ARG;


 typedef struct
   {
   int       Comm;
   int       rind;
   int       nrec;
   K_MREC    mrec;
   } _Z4ARG;


 typedef struct k_args
   {
   K_ARGS   *Forw;
   K_ARGS  **Head;
   K_PRIO    Prio;
   K_NODE    Srce;
   K_NODE    Dest;
   K_COMM    Comm;
   K_CREF    Ctxt;
   union
     {
     K_TICKS   ticks;
     K_TIMER  *timer;
     int       rcode;
     }       Time;
   union
     {
     _A1ARG    a1;
     _C1ARG    c1;
     _D1ARG    d1;
     _E1ARG    e1;
     _G1ARG    g1;
     _L1ARG    l1;
     _M1ARG    m1;
     _Q1ARG    q1;
     _Q2ARG    q2;
     _S1ARG    s1;
     _U1ARG    u1;
     _X1ARG    x1;
     _Z1ARG    z1;
     _Z2ARG    z2;
     _Z3ARG    z3;
     _Z4ARG    z4;
     }       Args;
   };


/* ----------------------------------------------------------------------*/
/* KERNEL OBJECT STRUCTURES */


 typedef struct
   {
   int        Nelms;
   int        Esize;
   char      *Base;
   char      *Free;
   K_ARGS    *Waiters;
   int        Nused;
   int        Hmark;
   int        Count;
   } MAP_STRUCT;


 typedef struct
   {
   int        Nelms;
   int        Esize;
   char      *Base;
   char      *Endp;
   char      *Enqp;
   char      *Deqp;
   K_ARGS    *Waiters;
   int        Nused;
   int        Hmark;
   int        Count;
   } QUE_STRUCT;


 typedef struct
   {
   K_ARGS    *Writers;
   K_ARGS    *Readers;
   int        Count;
   } MBX_STRUCT;


 typedef struct
   {
   K_TASK     Owner;
   int        Level;
   K_ARGS    *Waiters;
   int        Count;
   int        Confl;
   } RES_STRUCT;


 typedef struct
   {
   K_ARGS    *Waiters;
   int        Level;
   int        Count;
   } SEM_STRUCT;


 typedef struct
   {
   int        timer;
   int        action;
   K_TASK     task;
   } MON_STRUCT;


#endif
