/*
 ----------------------------------------------------------------------

   VIRTUOSO CLASSICO

   $Id: _stdio.h 1.3 94/05/16 11:00:08 FA REL3093 Locker: FA $

   Copyright (c) 1990-93
   Intelligent Systems International nv/sa

   Lindestraat 9, B-3210 Linden
   Belgium

   Tel +32 16.62.15.85
   Fax +32 16.62.15.84

 ----------------------------------------------------------------------
*/


/* _STDIO.H - standard input/output functions */

#ifndef _STDIO_H
#define _STDIO_H

#include <stdarg.h>

typedef int FILE;
typedef long fpos_t;

#ifndef __size_t
#define __size_t 1
typedef unsigned int size_t;	/* from <stddef.h> */
#endif

#ifndef NULL
#define NULL (void *)0		/* from <stddef.h> */
#endif

/* File pointers are really small integers */
#define stdin  (FILE *)1
#define stdout (FILE *)2
#define stderr (FILE *)3

/* Constants to be used as 3rd argument for "fseek" function */
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

/* Buffering types to be used as 3rd argument for "setvbuf" function */
#define _IOFBF 0
#define _IOLBF 1
#define _IONBF 2

/* Default buffer size use by "setbuf" function */
#define BUFSIZ 512

/* End-of-file constant definition */
#define EOF (-1)

/* Struct and constants for stat() and fstat () */
 struct stat
   {
   short st_dev;
   short st_ino;
   short st_mode;
   short st_nlink;
   int   st_uid;
   int   st_gid;
   short st_rdev;
   long  st_size;
   long  st_atime;
   long  st_mtime;
   long  st_ctime;
   };

#define S_IFMT   0xF000  /* file type mask */
#define S_IFDIR  0x4000  /* directory */
#define S_IFIFO  0x1000  /* FIFO special */
#define S_IFCHR  0x2000  /* character special */
#define S_IFBLK  0x3000  /* block special */
#define S_IFREG  0x8000  /* or just 0x0000, regular */
#define S_IREAD  0x0100  /* owner may read */
#define S_IWRITE 0x0080  /* owner may write */
#define S_IEXEC  0x0040  /* owner may execute <directory search> */


#ifndef STDIO_C

 extern int errno;

 extern FILE  *fopen (const char *path, const char *mode);
 extern FILE  *freopen (const char *path, const char *mode, FILE *stream);
 extern int    fclose (FILE *stream);

 extern int    fgetc (FILE *stream);
 extern int    fputc (int c, FILE *stream);
 extern int    ungetc (int c, FILE *stream);

 extern char  *fgets (char *string, int n, FILE *stream);
 extern int    fputs (const char *string, FILE *stream);

 extern char  *gets (char *string);
 extern int    puts (const char *string);

 extern size_t fread (void *ptr, size_t size, size_t nmembs, FILE *stream);
 extern size_t fwrite (const void *ptr, size_t size, size_t nmembs, FILE *stream);

 extern int    fgetpos (FILE *stream, fpos_t *pos);
 extern int    fsetpos (FILE *stream, const fpos_t *pos);

 extern int    feof (FILE *stream);
 extern int    ferror (FILE *stream);
 extern int    fflush (FILE *stream);

 extern int    fseek (FILE *stream, long int offset, int origin);
 extern long   ftell (FILE *stream);

 extern int    setvbuf (FILE *stream, char *buf, int mode, size_t size);

 extern int    rename (const char *old, const char *new);
 extern int    unlink (const char *name);
 extern int    fileno (FILE *stream);
 extern int    fstat (int filedesc, struct stat *info);
 extern int    stat (const char *name, struct stat *info);

#endif

#ifndef FPRINTF_C

 extern int fprintf  (FILE *F, const char *format, ...);
 extern int vfprintf (FILE *F, const char *format, va_list vargs);
 extern int printf   (const char *format, ...);
 extern int vprintf  (const char *format, va_list vargs);

#endif

#ifndef SPRINTF_C

 extern int sprintf  (char *s, const char *format, ...);
 extern int vsprintf (char *s, const char *format, va_list vargs);

#endif

#ifndef FSCANF_C

 extern int fscanf (FILE *F, const char *format, ...);
 extern int scanf (const char *format, ...);

#endif

#ifndef SSCANF_C

 extern int sscanf (char *s, const char *format, ...);

#endif

#define getc fgetc
#define putc fputc
#define getchar()  fgetc (stdin)
#define putchar(c) fputc (c,stdout)

#define rewind(s) fseek (s,0,SEEK_SET)
#define setbuf(s,b) setvbuf (s,b,_IOFBF,BUFSIZ)
#define remove(f) unlink(f)

#endif
